/*
 * MASC (Mobitex Asynchronous Communication Protocol)
 * Protocol Implementation for Linux
 * Copyright 1998 Kaz Kylheku <kaz@cafe.net>
 * This work is offered under the GNU Public License
 * as part of the Linux operating system.
 */

/*
 * $Id: mobitex_masc.h,v 1.12.2.3 1999/06/13 00:05:28 kaz Exp $
 * $Name: masc_1_9 $
 */

#ifndef MOBITEX_MASC_H
#define MOBITEX_MASC_H

#include <linux/tty.h>			/* for struct tty_struct	*/
#include <linux/netdevice.h>		/* for struct device		*/

#define N_MASC			8	/* Define these just in case	*/
#define ETH_P_MOBITEX		0x0015	/* the kernel doesn't have them	*/

#define MASC_NRUNIT		16
#define MASC_MAGIC		0x4d02	/* TODO: choose more wisely?	*/
#define MASC_MTU		510	/* 512 payload - protocol tag	*/
#define MASC_BUF_SIZE		2000	/* TODO: set the right value	*/
#define MASC_MAX_QUEUE		3	/* small downward queue		*/
#define MASC_MAX_DEV_QUEUE	3	/* network driver queue length	*/
#define MASC_MAX_MFRAME		2	/* maximum simultaneous frames	*/
					/* we stuff into the modem 	*/
#define MASC_ACK_TIMEOUT	10	/* seconds			*/
#define MASC_TRANS_TIMEOUT	60	/* seconds			*/
#define MASC_BAT_POLL		20	/* poll interval for battery	*/
					/* and signal strength		*/
#define MASC_INIT_TIMEOUT	4	/* seconds before sending INIT	*/
#define MASC_TURNOFF_TIMEOUT	10	/* modem turn off timeout	*/
#define MASC_RESPONSE_TIMEOUT	10	/* ioctl timeout for masc cmds	*/
#define MASC_MAX_LIFE		60	/* maximum packet life, secs	*/
#define MASC_MAX_AIRED_PACKETS	10	/* maximum packets in the air	*/
#define MASC_MAX_AVG_AIRTIME	20	/* seconds packet spends in air	*/

typedef struct masc_dev {
	int magic;
	char if_name[8];		/* masc0 through masc999	*/
	struct device dev_base;		/* device base class		*/
	struct masc_dev *volatile next;	/* links for keeping circular	*/
	struct masc_dev *volatile prev;	/* list of these structures	*/
	unsigned char *tx_buf;		/* buffer for outgoing masc pkt	*/
	unsigned char *rx_buf;		/* buffer for incoming masc pkt	*/
	unsigned char *tx_head;		/* pointer to next char to xmit	*/
	int rx_count;			/* chars in receive buffer	*/
	int tx_left;			/* chars remaining to xmit	*/
	int line;			/* which masc device this is	*/
	struct tty_struct *tty;		/* backpointer to tty		*/
	struct device *dev;		/* pointer to embedded dev	*/
	unsigned flg_occup : 1;		/* device structure is in use	*/
	unsigned flg_pushing : 1;	/* transmit pushing in progress	*/
	unsigned flg_toss : 1;		/* toss frame that had errors	*/
	unsigned flg_oflow : 1;		/* receive buffer overflow	*/
	unsigned flg_initack : 1;	/* expecting INIT frame ack	*/
	unsigned flg_initsent : 1;	/* we did receive INIT frame	*/
	unsigned flg_racks : 1;		/* RACK sent flag		*/
	unsigned flg_retran : 1;	/* Retransmit requestor		*/
	unsigned flg_tstop : 1;		/* Mobitex network stopped us	*/
	unsigned flg_nocontact : 1;	/* Lost contact with network	*/
	unsigned flg_haveman : 1;	/* Have the local MAN		*/
	unsigned flg_modem_off_w : 1;	/* modem off wanted		*/
	unsigned flg_modem_off : 1;	/* modem has turned off		*/
	unsigned flg_xmit_off_w : 1;	/* transmitter off wanted	*/
	unsigned flg_xmit_on_w : 1;	/* transmitter on wanted	*/
	unsigned flg_xmit_off : 1;	/* transmitter turned off	*/
	unsigned flg_xmit_on : 1;	/* transmitter turned on	*/
	volatile unsigned long tx_errors;	/* communication err counters	*/
	volatile unsigned long rx_errors;
	volatile unsigned long tx_packets;	/* count of transmitted packets	*/
	volatile unsigned long rx_packets;	/*   "   "  received     "	*/
	volatile unsigned long tx_dropped;	/* packets dropped by network	*/
	volatile unsigned long rx_dropped;	/* dropped received packets	*/
	volatile unsigned long tx_mailbox;	/* count of packets put in box	*/
	volatile unsigned long rx_mailbox;	/* count of packets from mbox	*/
	volatile unsigned long rx_notran;	/* NO_TRANSFER errors		*/
	volatile unsigned long rx_illegal;	/* ILLEGAL errors		*/
	volatile unsigned long rx_congest;	/* CONGEST errors		*/
	volatile unsigned long rx_errcount;	/* ERROR errors			*/
	volatile unsigned long rx_busy;		/* BUSY errors			*/
	struct sk_buff_head mpak_queue;	/* queue of down-bound M frames	*/
	struct sk_buff_head info_queue;	/* queue of info frames		*/
	struct sk_buff_head ctrl_queue;	/* queue of control frames	*/
	struct sk_buff_head ackw_queue;	/* info frame awaiting ack	*/
	struct sk_buff_head free_queue;	/* queue of free buffers	*/
	struct sk_buff_head fh_queue;	/* queue of frames awaiting FH	*/
	struct timer_list ack_timer;	/* timer node for ACK/NACK	*/
	struct timer_list bat_timer;	/* timer for Battery/Signal	*/
	struct timer_list off_timer;	/* timer for turning modem off	*/
	struct timer_list ini_timer;	/* timer for sending INIT frame */
	struct wait_queue *off_wait;	/* wait here for modem off	*/
	struct wait_queue *tx_off_wait;	/* wait here for xmitter off	*/
	struct wait_queue *tx_on_wait;	/* wait here for xmitter on	*/
	int seqno;			/* modulo 10 MASC seqno		*/
	int last_ack;			/* ack we last received		*/
	int send_ack;			/* ack we'll send: -1, 0, 1	*/
	int nack_count;			/* counts consecutive NACKs	*/
	volatile int batt;		/* battery level		*/
	volatile int sign;		/* signal strength		*/
	volatile unsigned char errhist[4];	/* history of errors		*/
	volatile int traf_hist[MASC_MAX_LIFE];	/* histogram model of traffic	*/
	volatile int traf_curr;			/* index into histogram		*/
	volatile int traf_count;		/* estimated packets in net	*/
	struct timer_list traf_timer;	/* drives histogram forward	*/
	long flags;
} masc_dev_t;

/*
 * Internal packet types
 */

#define MASC_TYPE(X)	((X) & 0xFF)	/* mask to get type		*/
#define MASC_CTRL	0x0100		/* MASC control frame		*/
#define MASC_INFO	0x0200		/* MASC info frame, non MPAK	*/
#define MASC_MPAK	0x0400		/* MASC MPAK frame		*/
#define MASC_INIT	(MASC_INFO|1)	/* MASC init info frame	  	*/
#define MASC_TXOFF	(MASC_INFO|2)	/* MASC transmit off command	*/
#define MASC_TXON	(MASC_INFO|3)	/* MASC transmit off command	*/

/*
 * Mobitex subscription byte flags
 */

#define MBTX_MAILBOX_F		0x01
#define MBTX_POSACK_F		0x02
#define MBTX_SENDLIST_F		0x04
#define MBTX_UNKNOWN_F		0x08
#define MBTX_RESERVE_F		0x10

/* Mobitex subscription byte traffic flags */

#define MBTX_TFC_MASK		0xE0U
#define MBTX_TFC_SHIFT		5
#define MBTX_OK			0
#define MBTX_FROM_MAIL		1	
#define MBTX_IN_MAIL		2
#define MBTX_NO_TRANSFER	3
#define MBTX_ILLEGAL		4
#define MBTX_CONGEST		5
#define MBTX_ERROR		6
#define MBTX_BUSY		7

/*
 * Subscription byte value used in outgoing packets; none of the above flags
 * are set, and the traffic state is set to zero
 */

#define MBTX_OUTGOING_FLAGS	0

/*
 * Packet type byte flags
 */

#define MBTX_EXTERN_F		0x20

/*
 * Packet classes
 */

#define MBTX_CLASS_MASK		0xC0
#define MBTX_CLASS_SHIFT	6
#define MBTX_PSUBCOM		0
#define MBTX_PSOSCOM		1
#define MBTX_DTESERV		3

/*
 * Type field within packet type byte.
 */

#define MBTX_TYPE_MASK		0x1F
#define MBTX_TYPE_SHIFT		0

/*
 * Packet types within class PSUBCOM
 */

#define MBTX_TEXT		1
#define MBTX_EXTPAK		1	/* EXTERN_F */
#define MBTX_DATA		2
#define MBTX_STATUS		3
#define MBTX_HPDATA		4	/* higher protocol data */

/*
 * Packet types within class DTESERV
 */

#define MBTX_LOGINREQ		1
#define MBTX_LOGINGRA		2
#define MBTX_LOGINREF		3
#define MBTX_LOGOUT		4
#define MBTX_LOGOUTORD		5
#define MBTX_ACTIVE		7
#define MBTX_INACTIVE		8
#define MBTX_VICESOSRX		13
#define MBTX_SOSRX		14
#define MBTX_GROUPLIST		15
#define MBTX_FLEXLIST		17
#define MBTX_TIME		20
#define MBTX_MODE		24

/*
 * Outgoing packet type byte
 * - class 0 (PSUBCOM)
 * - EXTERNAL_F false
 * - type 4 within PSUBCOM class (HPDATA)
 */

#define MBTX_OUTGOING_TYPE	0x04

#define MBTX_PACKET		5	/* Mobitex HPID for packet traffic */

#ifdef MASC_DEBUG
#define MASC_DBG_PRINTK(X) printk X
#else
#define MASC_DBG_PRINTK(X)
#endif
#endif
