/*
 * Turn on MASC tty discipline on a given tty
 * $Id: ttymasc.c,v 1.2.2.2 1999/06/13 00:05:28 kaz Exp $
 * $Name: masc_1_9 $
 */

#include <termios.h>
#include <sys/ioctl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include "mobitex_masc_ioctl.h"

#ifndef N_MASC
#define N_MASC 8
#endif

static int fd = -1;			/* terminal file descriptor	*/
static char *progname;
static char *ttname;

static void pexit(char *msg)
{
	fprintf(stderr, "%s: ", progname);
	perror(msg);
	exit(EXIT_FAILURE);
}

static void set_discipline(int which)
{
	int mascdisc = which;
	if (ioctl(fd, TIOCSETD, &mascdisc) < 0)
		pexit("ioctl(TIOCSETD)");
}

static void turnoff_modem(void)
{
	if (ioctl(fd, MASCIOCTURNOFF) < 0)
		perror("ioctl(MASCIOCTURNOFF)");
}

static void transmit_on(void)
{
	if (ioctl(fd, MASCIOCTXON) < 0)
		perror("ioctl(MASCIOCTXON)");
}

static void transmit_off(void)
{
	if (ioctl(fd, MASCIOCTXOFF) < 0)
		perror("ioctl(MASCIOCTXOFF)");
}

void interrupt(int sig)
{

}

int main(int argc, char **argv)
{
	sigset_t blocked;
	enum { setup_ldisc, tran_off, tran_on } oper = setup_ldisc;
	int dont_turn_off = 0;

	if ((argc != 2 && argc != 3 && argv[0])
			|| (argc == 3 && argv[1][0] != '-')) {
		fprintf(stderr, "usage: %s [-n01] <tty_name>\n", argv[0]);
		exit(EXIT_FAILURE);
	}

	progname = argv[0];
	ttname = (argc == 2) ? argv[1] : argv[2];

	if (argc == 3) {
		int i;
		for (i = 1; i < strlen(argv[1]); i++) {
			switch(argv[1][i]) {
			case 'n':
				dont_turn_off = 1;
				break;
			case '1':
				oper = tran_on;
				break;
			case '0':
				oper = tran_off;
				break;
			default:
				fprintf(stderr, "%s: uknown option -%c",
					 argv[0], (int) argv[1][1]);
				exit(EXIT_FAILURE);
			}

		}
	}


	fd = open(ttname, O_RDWR | O_NOCTTY);

	if (fd == -1)
		pexit(ttname);

	signal(SIGTERM, interrupt);
	signal(SIGINT, interrupt);

	set_discipline(N_MASC);

	/*
	 * we must keep the tty open
	 */

	sigemptyset(&blocked);

	switch (oper) {
	case setup_ldisc:
		sigsuspend(&blocked);
		if (!dont_turn_off)
			turnoff_modem();
		set_discipline(N_TTY);
		break;
	case tran_on:
		transmit_on();
		break;
	case tran_off:
		transmit_off();
		break;
	}

	return 0;
}
